# -*- coding: utf-8 -*- 
##############################################################################
#
# Copyright (c) 2008 Camtocamp SA
# @author JB Aubort
# $Id: $
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import wizard
import netsvc
import time
import datetime
import pooler
import pprint
import tools
from osv import osv
from mx import DateTime

# ------------------------------------
#              Main
# ------------------------------------

# Import selected holidays into the current timesheet
def _import(self, cr, uid, data, context):
    days_per_week = 5
    self.pool = pooler.get_pool(cr.dbname)
    ts = self.pool.get('hr_timesheet_sheet.sheet').browse(cr, uid, data['ids'])[0]
    
    # Get user Id and employee Id
    user_ids = self.pool.get('hr.employee').search(cr, uid, [('user_id','=',uid)])
    if not len(user_ids):
        raise wizard.except_wizard('Error !', 'No employee defined for your user !')
    else:
        user_id = uid
        employee_id = user_ids[0]
    if ts.state <>'draft':
        raise wizard.except_wizard('Error !', 'You can not modify a confirmed timesheet, please ask the manager !')
    
    # Get user worktime
    user = self.pool.get('hr.employee').browse(cr, uid, [employee_id])[0]
    if not user.regime:
        raise wizard.except_wizard('Error !', 'You must define a regime for this user ! (Workhours by week)')
    hours_per_day = user.regime / days_per_week
    
    # Get selected holidays
    holidays_ids = data['form']['holidays'][0][2]
    if len(holidays_ids)>0:
        holidays = self.pool.get('hr.holidays').browse(cr, uid, holidays_ids)
        errors = []
        for h in holidays:
            if h.holiday_status.analytic_account.id:
                analytic_account = h.holiday_status.analytic_account.id
            else:
                raise wizard.except_wizard('Error !', 'Holiday status "'+ str(h.holiday_status.name) +'" has no associated analytic account !')
            analytic_line = self.pool.get('hr.analytic.timesheet')
            attendance =  self.pool.get('hr.attendance')
            aa=self.pool.get('account.analytic.account').browse(cr, uid, analytic_account, context)
            if h.date_from < ts.date_from:
                h.date_from = ts.date_from+' 00:00:00'
            if h.date_to > ts.date_to:
                h.date_to = ts.date_to +' 23:59:59'
            nb_days = _get_number_days_between_dates(cr, uid, context, h.date_from, h.date_to)
            for day in range(nb_days):
                    datetime_current = (DateTime.strptime(h.date_from, '%Y-%m-%d %H:%M:%S') + DateTime.RelativeDateTime(days=day)).strftime('%Y-%m-%d')
                    # Import holiday only at week
                    day_of_the_week = DateTime.strptime(datetime_current, '%Y-%m-%d').iso_week[2]
                    if day_of_the_week!=6 and day_of_the_week!=7:
                        # Creation des lignes analytiques
                        if len(analytic_line.search(cr, uid, [('date','=',datetime_current),('name','=',h.name),('user_id','=',user_id)]))<1:
                            unit_id = analytic_line._getEmployeeUnit(cr, uid, context)
                            product_id = analytic_line._getEmployeeProduct(cr, uid, context)
                            general_account_id = _getGeneralAccount(self, cr, uid, context)
                            journal_id = _getAnalyticJournal(self, cr, uid, context)
                            holiday_day = {
                                'name': h.name or 'Vacances',
                                'date': datetime_current,
                                'unit_amount': hours_per_day,
                                'product_uom_id': unit_id,
                                'product_id': product_id,
                                'amount': hours_per_day,
                                'account_id': aa.id,
                                'to_invoice': aa.to_invoice.id,
                                'sheet_id': ts.id,
                                'general_account_id': general_account_id,
                                'journal_id':  journal_id,
                            }
                            analytic_line.create(cr, uid, holiday_day, context)
                        else:
                            errors.append('%s: There already is an analytic line.'%(str(datetime_current)))
                        
                        # Creation des attendances
                        if len(attendance.search(cr, uid, [('name','=',datetime_current),('employee_id','=',employee_id)]))<1:
                            hours = divmod(hours_per_day*60,60)
                            date_end=" %d:%d:00"%(hours[0],hours[1])
                            start = {
                                'name': str(datetime_current),
                                'action' : 'sign_in',
                                'employee_id' : employee_id,
                                'sheet_id':ts.id,
                            }
                            end = {
                                'name': DateTime.strptime(datetime_current+date_end,'%Y-%m-%d %H:%M:%S'),
                                'action' : 'sign_out',
                                'employee_id' : employee_id,
                                'sheet_id':ts.id,
                            }
                            attendance.create(cr, uid, start, context)
                            attendance.create(cr, uid, end, context)
                        else:
                            errors.append('%s: There already is an attendance.'%(str(datetime_current)))
        if len(errors)>0:
            errors_str = "\n".join(errors)
            raise wizard.except_wizard('Error !', errors_str)
    else:
        raise wizard.except_wizard('Information:', 'No holiday to import !')
    return {}
# End _import()

# Get General Account Id for the current user
def _getGeneralAccount(self, cr, uid, context):
    emp_obj = self.pool.get('hr.employee')
    emp_id = emp_obj.search(cr, uid, [('user_id', '=', context.get('user_id', uid))])
    if emp_id:
        emp = self.pool.get('hr.employee').browse(cr, uid, emp_id[0], context=context)
        if bool(emp.product_id):
            a =  emp.product_id.product_tmpl_id.property_account_expense.id
            if not a:
                a = emp.product_id.categ_id.property_account_expense_categ.id
            if a:
                return a
    return 1 #FIXME: What could be the default value ?
# End _getGeneralAccount()

# Get Analytic Journal for current user
def _getAnalyticJournal(self, cr, uid, context):
    emp_obj = self.pool.get('hr.employee')
    emp_id = emp_obj.search(cr, uid, [('user_id', '=', context.get('user_id', uid))])
    if emp_id:
        emp = self.pool.get('hr.employee').browse(cr, uid, emp_id[0], context=context)
        if emp.journal_id:
            return emp.journal_id.id
    return 1 #FIXME: What could be the default value ?
# End _getAnalyticJournal()

# Get number of date betweek two dates and return the integer
def _get_number_days_between_dates(cr, uid, context, date_from, date_to):
    datetime_from=DateTime.strptime(date_from, '%Y-%m-%d %H:%M:%S')
    datetime_to=DateTime.strptime(date_to, '%Y-%m-%d %H:%M:%S')
    difference=datetime_to+1-datetime_from
##    weeks, days = divmod(difference.days, 7)
##    return int(days)
    return int(difference.days)
# End _get_number_days_between_dates()

# Get holidays within the current timesheet dates and return them as a tuple of ids
def _get_holidays(self, cr, uid, data, context):
    self.pool = pooler.get_pool(cr.dbname)
    ts = self.pool.get('hr_timesheet_sheet.sheet').browse(cr, uid, data['ids'])[0]
    date_from = ts.date_from+' 00:00:00'
    date_to = ts.date_to+' 23:59:59'
    req = ("select id,date_from,date_to,name from hr_holidays where\
    (\
        ((date_from<='%s' and date_to>='%s' and date_to<='%s') or\
        (date_from>='%s' and date_from<='%s' and date_to>= '%s') or\
        (date_from>='%s' and date_from<='%s' and date_to>='%s' and date_to<='%s') or\
        (date_from<='%s' and date_to>='%s')) and user_id='%i' and state='validate'\
    )"
    %(
        date_from, date_from, date_to,
        date_from, date_to, date_to,
        date_from, date_to, date_from, date_to,
        date_from, date_to, uid
    ))
    cr.execute(req)
    holiday = cr.fetchall()
    if len(holiday) > 0:
        holiday_ids = [] 
        analytic_line = self.pool.get('hr.analytic.timesheet')
        for h in holiday:
            check = 1
            if h[1] < ts.date_from:
                h_date_from = date_from
            else:
                h_date_from = h[1]
            if h[2] > ts.date_to:
                h_date_to = date_to
            else:
                h_date_to = h[2]
            nb_days = _get_number_days_between_dates(cr,uid,context,h_date_from,h_date_to)   
            for day in range(nb_days):
                datetime_current = (DateTime.strptime(h_date_from, '%Y-%m-%d %H:%M:%S') + DateTime.RelativeDateTime(days=day)).strftime('%Y-%m-%d')
                if len(analytic_line.search(cr, uid, [('date','=',datetime_current),('name','=', h[3]),('user_id','=',uid)]))>=1:
                    check = 0
            if check:
                holiday_ids.append(h[0])
        if len(holiday_ids)>0:
            return holiday_ids
        else:
            raise wizard.except_wizard('Information:', 'Holiday already imported !')
    else:
        raise wizard.except_wizard('Information:', 'No holiday for the current timesheet !')
    return {}
# End _get_holidays()

# ------------------------------------
#             Forms
# ------------------------------------
init_form = '''<?xml version="1.0"?>
<form string="Holiday import">
    <separator string="Select holidays to import:" colspan="4" />
    <field name="holidays" nolabel="1" />
</form>'''

_update_arch = '''<?xml version="1.0"?>
<form string="Holiday import">
    <label string="Holidays imported !" colspan="4"/>
</form>'''

# ------------------------------------
#        Wizard class
# ------------------------------------
class wizard_report(wizard.interface):
    
    def _set_holidays(self, cr, uid, data, context):
        self.states['init']['result']['fields']={
            'holidays': {'string':'Holidays', 'type':'many2many', 'relation':'hr.holidays', 'domain':[('state', '=', 'validate'),('user_id','=',uid)]},
        }
        return {
            'holidays':_get_holidays(self, cr, uid, data, context),
        }
    # End _set_holidays()

    states = {
        'init': {
            'actions': [_set_holidays],
            'result': {'type': 'form', 'arch': init_form, 'state':[ ('end','Cancel','gtk-cancel'),('import','Import','gtk-ok')]}
        },
        'import': {
            'actions': [_import],
            'result': {'type': 'form', 'arch':_update_arch, 'fields': {},  'state':[('end', 'Close')]}
        }
    }
wizard_report('c2c.import.holidays')
